# flake8-in-file-ignores: noqa: F401,F403,F405

# Copyright (c) 1994 Adam Karpierz
# SPDX-License-Identifier: Zlib

__all__ = (
    'is_windows', 'is_linux', 'is_macos', 'defined',
    'DLL_PATH', 'DLL', 'dlclose', 'CFUNC',
)

from utlx.platform import *
if is_windows:  # pragma: no cover
    from .windows import DLL_PATH, DLL, dlclose, CFUNC
elif is_linux:  # pragma: no cover
    from .linux import DLL_PATH, DLL, dlclose, CFUNC
elif is_macos:  # pragma: no cover
    from .macos import DLL_PATH, DLL, dlclose, CFUNC
else:  # pragma: no cover
    raise ImportError("Unsupported platform")



# Copyright (c) 1994 Adam Karpierz
# SPDX-License-Identifier: Zlib

from ._platform import DLL_PATH, DLL
from ._platform import dlclose  # noqa: F401

try:
    dll = DLL(DLL_PATH)
except OSError as exc:  # pragma: no cover
    raise exc
except Exception as exc:  # pragma: no cover
    raise OSError(f"{exc}") from None



# flake8-in-file-ignores: noqa: E402

# Copyright (c) 1994 Adam Karpierz
# SPDX-License-Identifier: Zlib

import ctypes
import sysconfig
from pathlib import Path
from functools import partial

__all__ = ('DLL_PATH', 'DLL', 'dlclose', 'CFUNC')

this_dir = Path(__file__).resolve().parent
dll_suff = sysconfig.get_config_var("EXT_SUFFIX") or ".pyd"

DLL_PATH = str(this_dir.parent/("crc" + dll_suff))

from ctypes  import CDLL as _DLL
from _ctypes import dlclose  # type: ignore[attr-defined]
from ctypes  import CFUNCTYPE as CFUNC

DLL = partial(_DLL, mode=ctypes.RTLD_GLOBAL)



# Copyright (c) 1994 Adam Karpierz
# SPDX-License-Identifier: Zlib

import unittest
import itertools

import crc


class CrcTestCase(unittest.TestCase):

    def setUp(self):

        self.check_seq = b"123456789"

        self.crc_models = (crc.model_t * (4 + 1))(
            # name                width  poly        init        refin  refout xorout      check #
            #------------------------------------------------------------------------------------#
            crc.model(b"XXX-32", 32, 0x04C11DB7, 0xFFFFFFFF, True,  True,  0xFFFFFFFF, 0xCBF43926),
            crc.model(b"YYY-32", 32, 0x04C11DB7, 0xFFFFFFFF, False, False, 0xFFFFFFFF, 0xFC891918),
            crc.model(b"ZZZ-32", 32, 0x04C11DB7, 0xFFFFFFFF, False, True,  0xFFFFFFFF, 0x1898913F),
            crc.model(b"RRR-32", 32, 0x04C11DB7, 0xFFFFFFFF, True,  False, 0xFFFFFFFF, 0x649C2FD3),
        )

        # Hacks to initialize predefined models table (only needed
        # when we iterate over the table by the index).
        crc.predefined_model_by_name(b"")

        self.crc_predefined_model_names = [crc_model.name.decode("utf-8")
                                           for crc_model in crc.predefined_models]
        self.crc_model_names = []
        for crc_model in self.crc_models:  # pragma: no cover
            if crc_model.width == 0: break
            self.crc_model_names.append(crc_model.name.decode("utf-8"))

    def test_predefined_models(self):
        """Test of predefined CRC models"""
        for crc_model in crc.predefined_models:
            crc_result = crc.init(crc_model)
            crc_result = crc.update(crc_model, self.check_seq, 9, crc_result)
            crc_result = crc.final(crc_model, crc_result)
            self.assertEqual(crc_result, crc_model.check)
            print("{:>22}: {:016X}".format(crc_model.name.decode("utf-8"), crc_result))
        print()

    def test_user_models(self):
        """Test of user-defined CRC models"""
        for crc_model in self.crc_models:  # pragma: no cover
            if crc_model.width == 0: break
            crc_result = crc.init(crc_model)
            crc_result = crc.update(crc_model, self.check_seq, 9, crc_result)
            crc_result = crc.final(crc_model, crc_result)
            self.assertEqual(crc_result, crc_model.check)
            print("{:>22}: {:016X}".format(crc_model.name.decode("utf-8"), crc_result))
        print()

    def test_predefined_models_by_name(self):
        """Test of predefined CRC models by model name"""
        for name in self.crc_predefined_model_names:
            crc_model = crc.predefined_model_by_name(name.encode("utf-8"))[0]
            self.assertEqual(name, crc_model.name.decode("utf-8"))
            crc_result = crc.init(crc_model)
            crc_result = crc.update(crc_model, self.check_seq, 9, crc_result)
            crc_result = crc.final(crc_model, crc_result)
            self.assertEqual(crc_result, crc_model.check)
            print("{:>22}: {:016X}".format(crc_model.name.decode("utf-8"), crc_result))
        print()

    def test_user_models_by_name(self):
        """Test of user-defined CRC models by model name"""
        for name in self.crc_model_names:
            crc_model = crc.model_by_name(name.encode("utf-8"), self.crc_models)[0]
            self.assertEqual(name, crc_model.name.decode("utf-8"))
            crc_result = crc.init(crc_model)
            crc_result = crc.update(crc_model, self.check_seq, 9, crc_result)
            crc_result = crc.final(crc_model, crc_result)
            self.assertEqual(crc_result, crc_model.check)
            print("{:>22}: {:016X}".format(crc_model.name.decode("utf-8"), crc_result))
        print()



# Copyright (c) 1994 Adam Karpierz
# SPDX-License-Identifier: Zlib

"""Public Python API of CRC package."""

from .__about__ import * ; del __about__  # type: ignore[name-defined]  # noqa

from ._crc import * ; del _crc  # type: ignore[name-defined]  # noqa



# flake8-in-file-ignores: noqa: E402,N813,N814

# Copyright (c) 1994 Adam Karpierz
# SPDX-License-Identifier: Zlib

from typing import Any
import ctypes
import sysconfig
from pathlib import Path

__all__ = ('DLL_PATH', 'DLL', 'dlclose', 'CFUNC')

this_dir = Path(__file__).resolve().parent
dll_suff = sysconfig.get_config_var("EXT_SUFFIX") or ".pyd"

DLL_PATH = str(this_dir.parent/("crc" + dll_suff))

from ctypes import WinDLL as _DLL
try:
    from _ctypes import FreeLibrary as dlclose
except ImportError:  # pragma: no cover
    dlclose = lambda handle: None
from ctypes import CFUNCTYPE as CFUNC

def DLL(*args: Any, **kwargs: Any) -> ctypes.CDLL:
    import os
    with os.add_dll_directory(os.path.dirname(args[0])):
        return _DLL(*args, **kwargs)
